@@ -1,15 +1,16 @@
 import datetime
-import difflib
 import json
 import pathlib
 import random
 import re
 import secrets
 import string
-import unicodedata
-from typing import Iterable, Sequence, Type, overload
+from typing import Iterable, Type, overload
 
-from flanautils import constants
+import jellyfish
+import unicodedata
+
+from flanautils import constants, iterables
 from flanautils.models.ratio_match import RatioMatch
 
 
@@ -21,13 +22,68 @@ def cartesian_product_string_matching(a_text: str | Iterable[str], b_text: str |
 
     a_words = a_text.split() if isinstance(a_text, str) else a_text
     b_words = b_text.split() if isinstance(b_text, str) else b_text
-    return {a_word: matches for a_word in a_words if (matches := {b_word: ratio for b_word in b_words if (ratio := matching_ratio(a=a_word, b=b_word)) >= min_ratio})}
+    return {a_word: matches for a_word in a_words if (matches := {b_word: ratio for b_word in b_words if (ratio := jellyfish.jaro_winkler_similarity(a_word, b_word)) >= min_ratio})}
 
 
-def find_jsons(text: str) -> list[dict]:
-    """Find all well formatted JSONs in the text and return them in dictionaries."""
+def cast_number(text: str, raise_exception=True) -> int | float | str:
+    """
+    Try to cast a string to a number.
 
-    text = path_to_text(text)
+    If raise_exception=False (True by default), returns the input as it is.
+    """
+
+    try:
+        return int(text)
+    except ValueError:
+        try:
+            return float(text)
+        except ValueError:
+            if raise_exception:
+                raise
+            else:
+                return text
+
+
+def find_coordinates(text: str) -> tuple[float, float] | list[tuple[float, float]] | None:
+    """
+    Find coordinates in string.
+
+    - If one pair of numbers are matched, returns (latitude, longitude)
+    - If more of one pair of numbers are matched, returns [(latitude, longitude), ...]
+    - If not match, returns None
+
+    >>> find_coordinates('holaaaa 85.1, 44.3515')
+    (85.1, 44.3515)
+    >>> find_coordinates('ey que pasa+654.1464;,;+ + +-456.4164hola')
+    (654.1464, -456.4164)
+    >>> find_coordinates('ey que pasa+654.1464;,;+ + -+456.4164hola    40+5.5')
+    [(654.1464, 456.4164), (40.0, 5.5)]
+    >>> find_coordinates('ey que pasa')
+
+    """
+
+    results = re.findall(r'[-+\d.]+[,;\s+-]+[-+\d.]+', replace(text, {'-': ' -', '+': ' +'}))
+
+    formatted_results = []
+    for result in results:
+        try:
+            words = re.split(r'[,;\s+]+', result)
+            latitude, longitude = iterables.filter(words, condition=lambda e: isinstance(e, int | float), cast_numbers=True)
+        except ValueError:
+            continue
+        formatted_results.append((float(latitude), float(longitude)))
+
+    match formatted_results:
+        case [single]:
+            return single
+        case [_, *_]:
+            return formatted_results
+
+
+def find_jsons(text: str | pathlib.Path) -> list[dict]:
+    """Find all well formatted JSONs in a string or in a pathlib.Path and return them in dictionaries."""
+
+    text = read_file(text) or text
 
     jsons = []
     position = 0
@@ -38,10 +94,11 @@ def find_jsons(text: str) -> list[dict]:
 
 
 def find_environment_variables(text: str | pathlib.Path) -> dict:
-    """Looks for environment variables in the text in .env format (key=value)."""
+    """Looks for environment variables in a string or in a pathlib.Path in .env format (key=value)."""
 
+    text = read_file(text) or text
     # noinspection PyTypeChecker
-    return dict(line.split('=', maxsplit=1) for line in path_to_text(text).splitlines() if '=' in line)
+    return dict(line.split('=', maxsplit=1) for original_line in text.splitlines() if '=' in (line := original_line.strip()) and not line.startswith('#'))
 
 
 def join_last_separator(elements: Iterable, separator: str, last_separator: str, final_char='') -> str:
@@ -63,29 +120,6 @@ def join_last_separator(elements: Iterable, separator: str, last_separator: str,
     return f'{separator.join(elements[:-1])}{last_separator}{elements[-1]}{final_char}'
 
 
-def matching_ratio(a: Sequence[str], b: Sequence[str], original_index_weight=0.8, custom_index_weight=0.2) -> float:
-    """
-    Compare two sequences and return the score.
-
-    It is made up of two different indices. The first uses difflib.SequenceMatcher and the second, a custom one that
-    obtains a ratio of equal elements over the total number of elements.
-
-    The weights of both indices can be configured by arguments.
-    """
-
-    if custom_index_weight > 0:
-        match_count = 0
-        i = None
-        for i, (a_element, b_element) in enumerate(zip(a, b), start=1):
-            if a_element == b_element:
-                match_count += 1
-        custom_matching_ratio = match_count / i if i else 0.5
-    else:
-        custom_matching_ratio = 0
-
-    return original_index_weight * difflib.SequenceMatcher(a=a, b=b).ratio() + custom_index_weight * custom_matching_ratio
-
-
 def numbers_to_words(number: int, language='es') -> str:
     """
     Convert an integer into its textual representation according to the language.
@@ -134,16 +168,6 @@ def numbers_to_words(number: int, language='es') -> str:
         raise NotImplementedError('not implemented for that language')
 
 
-def path_to_text(path: str | pathlib.Path) -> str:
-    """
-    Returns the content of the file as text.
-
-    Returns the path argument as is if path is not a valid path string or pathlib.Path.
-    """
-
-    return read_file(path) or path
-
-
 @overload
 def random_string(min_len: int, max_len: int, letters=True, numbers=True, n_spaces=0) -> str:
     pass
@@ -196,67 +220,48 @@ def read_file(path: str | pathlib.Path) -> str | None:
                     return file.read()
 
 
-def remove_accents(text: str, ignore=('ñ', 'ç')) -> str:
+def remove_accents(text: str, ignore: Iterable = ('ñ', 'ç')) -> str:
     """
     Encode the string to remove the accents from non ascii characters.
 
-    Ignore the strings contained in ignore argument. By default ignore=('ñ', 'ç').
+    Ignore the characters contained in ignore argument. By default ignore=('ñ', 'ç').
 
     >>> remove_accents('aáeèiîoöuuºªçñ')
     'aaeeiioouuçñ'
-    >>> remove_accents('Mañana iba a salir pero el otro día iba por la calle y casi me atropella un camión que iba muy rapido.')
+    >>> remove_accents('Mañana iba a salir pero el otro día iba por la calle y casi me atropella un camión que iba muy rápido.')
     'Mañana iba a salir pero el otro dia iba por la calle y casi me atropella un camion que iba muy rapido.'
     """
 
     return ''.join(char if char in ignore else unicodedata.normalize('NFD', char).encode('ascii', 'ignore').decode() for char in text)
 
 
-@overload
-def str_to_class(class_names: str | Type, globals_: dict) -> Type:
-    pass
-
-
-@overload
-def str_to_class(class_names: Iterable[str | Type], globals_: dict) -> tuple[Type, ...]:
-    pass
-
-
-def str_to_class(class_names: str | Type | Iterable[str | Type], globals_: dict) -> Type | tuple[Type, ...]:
-    """Converts the class or classes names into Python Types found in globals_."""
-
-    globals_ = globals_ or {}
-    if isinstance(class_names, str):
-        return globals_[class_names]
-    else:
-        return tuple(globals_[class_name] for class_name in class_names)
-
-
-def sum_numbers_in_text(text: str, language='es') -> int:
+def remove_symbols(text: str, ignore: Iterable = (), replace_with='') -> str:
     """
-    Add all the existing numbers in a text, whether they are in numerical or textual form.
+    Remove the typical symbols from the string.
 
-    >>> sum_numbers_in_text('Uno más dos. Y luego cuarenta y cuatro y cuarenta y 2 y 10 y 1 más 7.')
-    107
+    Ignore the characters contained in ignore argument. By default ignore=().
+
+    Symbols are replaced with replace_with argument. By default replace_with=''.
+
+    >>> remove_symbols('¿No vás muy rápido? Yo creo que sí.')
+    'No vás muy rápido Yo creo que sí'
+    >>> remove_symbols('¿No vás muy rápido? Yo creo que sí.', ignore=('?',), replace_with='8')
+    '8No vás muy rápido? Yo creo que sí8'
     """
 
-    words = translate(text, {symbol: None for symbol in constants.SYMBOLS}).split()
-    n = 0
-    for word in words:
-        try:
-            n += int(word)
-        except ValueError:
-            pass
-
-    return n + words_to_numbers(text, language=language)
+    return ''.join((char if char in ignore else replace_with) if char in constants.SYMBOLS else char for char in text)
 
 
-def translate(text: str, replacements: dict) -> str:
+replace_symbols = remove_symbols
+
+
+def replace(text: str, replacements: dict) -> str:
     """
     Returns a copy of text with the replacements applied.
 
-    >>> translate('abc.-.', {'.': '*', '-': '<===>'})
+    >>> replace('abc.-.', {'.': '*', '-': '<===>'})
     'abc*<===>*'
-    >>> translate('Hola1 que2 ase3', {'Hola': 'Hi', 'que': 'ase', 'ase': None})
+    >>> replace('Hola1 que2 ase3', {'Hola': 'Hi', 'que': 'ase', 'ase': None})
     'Hi1 ase2 3'
     """
 
@@ -286,6 +291,49 @@ def translate(text: str, replacements: dict) -> str:
     return ''.join(result)
 
 
+translate = replace
+
+
+@overload
+def str_to_class(class_names: str | Type, globals_: dict) -> Type:
+    pass
+
+
+@overload
+def str_to_class(class_names: Iterable[str | Type], globals_: dict) -> tuple[Type, ...]:
+    pass
+
+
+def str_to_class(class_names: str | Type | Iterable[str | Type], globals_: dict) -> Type | tuple[Type, ...]:
+    """Converts the class or classes names into Python Types found in globals_."""
+
+    globals_ = globals_ or {}
+    if isinstance(class_names, str):
+        return globals_[class_names]
+    else:
+        return tuple(globals_[class_name] for class_name in class_names)
+
+
+def sum_numbers_in_text(text: str, language='es') -> int:
+    """
+    Add all the existing numbers in a text, whether they are in numerical or textual form.
+
+    >>> sum_numbers_in_text('Uno más dos. Y luego cuarenta y cuatro y cuarenta y 2 y 10 y 1 más 7.')
+    107
+    >>> sum_numbers_in_text('0.1 + uno más dos, 5.2 -1.1 mas 5.3 -5.8 ')
+    6.7
+    """
+
+    n = 0
+    for word in text.split():
+        try:
+            n += cast_number(word.strip('.'))
+        except ValueError:
+            pass
+
+    return n + words_to_numbers(text, language=language)
+
+
 def words_to_numbers(text: str, ignore_no_numbers=True, language='es') -> int:
     """
     Convert all numbers in textual representation, according to the language, into an integer and return the sum of
@@ -300,18 +348,18 @@ def words_to_numbers(text: str, ignore_no_numbers=True, language='es') -> int:
     if language == 'es':
         number_words_es = constants.NUMBER_WORDS[language]
         text = re.sub(r'(([ei]nt[aeio])|(ec))', r'\1 ', text)
-        text = translate(text, {symbol: None for symbol in constants.SYMBOLS} | {'y': ' '})
+        text = replace(text, {symbol: None for symbol in constants.SYMBOLS} | {'y': ' '})
         words = text.lower().split()
         n = 0
         sign_ = 1
         for word in words:
-            if matching_ratio(a=word, b=number_words_es['-']) >= constants.NUMBERS_RATIO_MATCHING:
+            if jellyfish.jaro_winkler_similarity(word, number_words_es['-']) >= constants.NUMBERS_RATIO_MATCHING:
                 sign_ = -1
                 continue
 
             word_possible_matches = []
             for number_word in number_words_es.values():
-                ratio = matching_ratio(a=word, b=number_word)
+                ratio = jellyfish.jaro_winkler_similarity(word, number_word)
                 if ratio >= constants.NUMBERS_RATIO_MATCHING:
                     word_possible_matches.append(RatioMatch(number_words_es[number_word], ratio))
 
@@ -337,25 +385,25 @@ def words_to_time(text: str, language='es') -> datetime.timedelta:
         delta_time = datetime.timedelta()
         n = 0
         for word in text.split():
-            if matching_ratio(a=word, b='segundo') >= constants.TIME_UNITS_RATIO_MATCHING:
+            if jellyfish.jaro_winkler_similarity(word, 'segundo') >= constants.TIME_UNITS_RATIO_MATCHING:
                 delta_time += datetime.timedelta(seconds=n)
                 n = 0
-            elif matching_ratio(a=word, b='minuto') >= constants.TIME_UNITS_RATIO_MATCHING or matching_ratio(a=word, b='min') >= constants.TIME_UNITS_RATIO_MATCHING:
+            elif jellyfish.jaro_winkler_similarity(word, 'minuto') >= constants.TIME_UNITS_RATIO_MATCHING or jellyfish.jaro_winkler_similarity(word, 'min') >= constants.TIME_UNITS_RATIO_MATCHING:
                 delta_time += datetime.timedelta(minutes=n)
                 n = 0
-            elif matching_ratio(a=word, b='hora') >= constants.TIME_UNITS_RATIO_MATCHING:
+            elif jellyfish.jaro_winkler_similarity(word, 'hora') >= constants.TIME_UNITS_RATIO_MATCHING:
                 delta_time += datetime.timedelta(hours=n)
                 n = 0
-            elif matching_ratio(a=word, b='dia') >= constants.TIME_UNITS_RATIO_MATCHING:
+            elif jellyfish.jaro_winkler_similarity(word, 'dia') >= constants.TIME_UNITS_RATIO_MATCHING:
                 delta_time += datetime.timedelta(days=n)
                 n = 0
-            elif matching_ratio(a=word, b='semana') >= constants.TIME_UNITS_RATIO_MATCHING:
+            elif jellyfish.jaro_winkler_similarity(word, 'semana') >= constants.TIME_UNITS_RATIO_MATCHING:
                 delta_time += datetime.timedelta(weeks=n)
                 n = 0
-            elif matching_ratio(a=word, b='mes') >= constants.TIME_UNITS_RATIO_MATCHING:
+            elif jellyfish.jaro_winkler_similarity(word, 'mes') >= constants.TIME_UNITS_RATIO_MATCHING:
                 delta_time += datetime.timedelta(weeks=n * 4.34524)
                 n = 0
-            elif matching_ratio(a=word, b='año') >= constants.TIME_UNITS_RATIO_MATCHING:
+            elif jellyfish.jaro_winkler_similarity(word, 'año') >= constants.TIME_UNITS_RATIO_MATCHING:
                 delta_time += datetime.timedelta(weeks=n * 52.1429)
                 n = 0
             else:
